@@ -1,11 +1,11 @@
 use std::{env, io};
 
-use bytes::BytesMut;
 use futures::StreamExt;
 use log::info;
 use ntex::http::header::HeaderValue;
 use ntex::http::{HttpService, Request, Response};
 use ntex::server::Server;
+use ntex::{time::Seconds, util::BytesMut};
 
 #[ntex::main]
 async fn main() -> io::Result<()> {
@@ -15,8 +15,8 @@ async fn main() -> io::Result<()> {
     Server::build()
         .bind("echo", "127.0.0.1:8080", || {
             HttpService::build()
-                .client_timeout(1000)
-                .disconnect_timeout(1000)
+                .client_timeout(Seconds(1))
+                .disconnect_timeout(Seconds(1))
                 .finish(|mut req: Request| async move {
                     let mut body = BytesMut::new();
                     while let Some(item) = req.payload().next().await {
