@@ -37,7 +37,7 @@ impl<T: Address + 'static> RustlsConnector<T> {
         Connect<T>: From<U>,
     {
         let req = Connect::from(message);
-        let host = req.host().to_string();
+        let host = req.host().split(':').next().unwrap().to_owned();
         let conn = self.connector.call(req);
         let config = self.config.clone();
 
@@ -109,7 +109,7 @@ mod tests {
     #[crate::rt_test]
     async fn test_rustls_connect() {
         let server = crate::server::test_server(|| {
-            crate::fn_service(|_| async { Ok::<_, ()>(()) })
+            crate::service::fn_service(|_| async { Ok::<_, ()>(()) })
         });
 
         let mut config = ClientConfig::new();
