@@ -1,7 +1,5 @@
 //! Test helpers to use during testing.
-use std::{convert::TryFrom, io, net, str::FromStr, sync::mpsc, thread, time};
-
-use bytes::Bytes;
+use std::{convert::TryFrom, io, net, str::FromStr, sync::mpsc, thread};
 
 #[cfg(feature = "cookie")]
 use coo_kie::{Cookie, CookieJar};
@@ -9,6 +7,7 @@ use coo_kie::{Cookie, CookieJar};
 use crate::codec::{AsyncRead, AsyncWrite, Framed};
 use crate::rt::{net::TcpStream, System};
 use crate::server::{Server, StreamServiceFactory};
+use crate::{time::Millis, time::Seconds, util::Bytes};
 
 use super::client::error::WsClientError;
 use super::client::{Client, ClientRequest, ClientResponse, Connector};
@@ -244,20 +243,18 @@ pub fn server<F: StreamServiceFactory<TcpStream>>(factory: F) -> TestServer {
                     .set_alpn_protos(b"\x02h2\x08http/1.1")
                     .map_err(|e| log::error!("Cannot set alpn protocol: {:?}", e));
                 Connector::default()
-                    .timeout(time::Duration::from_millis(30000))
+                    .timeout(Millis(30_000))
                     .openssl(builder.build())
                     .finish()
             }
             #[cfg(not(feature = "openssl"))]
             {
-                Connector::default()
-                    .timeout(time::Duration::from_millis(30000))
-                    .finish()
+                Connector::default().timeout(Millis(30_000)).finish()
             }
         };
 
         Client::build()
-            .timeout(time::Duration::from_millis(30000))
+            .timeout(Seconds(30))
             .connector(connector)
             .finish()
     };
