@@ -1,20 +1,15 @@
 use std::task::{Context, Poll};
-use std::{
-    cell::RefCell, convert::Infallible, future::Future, marker, pin::Pin, time::Duration,
-};
+use std::{cell::Cell, convert::Infallible, marker, time::Duration, time::Instant};
 
-use crate::rt::time::{sleep_until, Instant, Sleep};
+use crate::time::{now, sleep, Millis, Sleep};
 use crate::{util::Ready, Service, ServiceFactory};
 
-use super::time::{LowResTime, LowResTimeService};
-
 /// KeepAlive service factory
 ///
 /// Controls min time between requests.
 pub struct KeepAlive<R, E, F> {
     f: F,
-    ka: Duration,
-    time: LowResTime,
+    ka: Millis,
     _t: marker::PhantomData<(R, E)>,
 }
 
@@ -26,10 +21,9 @@ where
     ///
     /// ka - keep-alive timeout
     /// err - error factory function
-    pub fn new(ka: Duration, time: LowResTime, err: F) -> Self {
+    pub fn new(ka: Millis, err: F) -> Self {
         KeepAlive {
             ka,
-            time,
             f: err,
             _t: marker::PhantomData,
         }
@@ -44,7 +38,6 @@ where
         KeepAlive {
             f: self.f.clone(),
             ka: self.ka,
-            time: self.time.clone(),
             _t: marker::PhantomData,
         }
     }
@@ -63,41 +56,30 @@ where
     type Future = Ready<Self::Service, Self::InitError>;
 
     fn new_service(&self, _: ()) -> Self::Future {
-        Ready::Ok(KeepAliveService::new(
-            self.ka,
-            self.time.timer(),
-            self.f.clone(),
-        ))
+        Ready::Ok(KeepAliveService::new(self.ka, self.f.clone()))
     }
 }
 
 pub struct KeepAliveService<R, E, F> {
     f: F,
-    ka: Duration,
-    time: LowResTimeService,
-    inner: RefCell<Inner>,
+    dur: Millis,
+    sleep: Sleep,
+    expire: Cell<Instant>,
     _t: marker::PhantomData<(R, E)>,
 }
 
-struct Inner {
-    delay: Pin<Box<Sleep>>,
-    expire: Instant,
-}
-
 impl<R, E, F> KeepAliveService<R, E, F>
 where
     F: Fn() -> E,
 {
-    pub fn new(ka: Duration, time: LowResTimeService, f: F) -> Self {
-        let expire = Instant::from_std(time.now() + ka);
+    pub fn new(dur: Millis, f: F) -> Self {
+        let expire = Cell::new(now());
+
         KeepAliveService {
             f,
-            ka,
-            time,
-            inner: RefCell::new(Inner {
-                expire,
-                delay: Box::pin(sleep_until(expire)),
-            }),
+            dur,
+            expire,
+            sleep: sleep(dur),
             _t: marker::PhantomData,
         }
     }
@@ -113,17 +95,16 @@ where
     type Future = Ready<R, E>;
 
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let mut inner = self.inner.borrow_mut();
-
-        match Pin::new(&mut inner.delay).poll(cx) {
+        match self.sleep.poll_elapsed(cx) {
             Poll::Ready(_) => {
-                let now = Instant::from_std(self.time.now());
-                if inner.expire <= now {
+                let now = now();
+                let expire = self.expire.get() + Duration::from(self.dur);
+                if expire <= now {
                     Poll::Ready(Err((self.f)()))
                 } else {
-                    let expire = inner.expire;
-                    inner.delay.as_mut().reset(expire);
-                    let _ = Pin::new(&mut inner.delay).poll(cx);
+                    let expire = expire - now;
+                    self.sleep.reset(Millis(expire.as_millis() as u64));
+                    let _ = self.sleep.poll_elapsed(cx);
                     Poll::Ready(Ok(()))
                 }
             }
@@ -132,7 +113,7 @@ where
     }
 
     fn call(&self, req: R) -> Self::Future {
-        self.inner.borrow_mut().expire = Instant::from_std(self.time.now() + self.ka);
+        self.expire.set(now());
         Ready::Ok(req)
     }
 }
@@ -140,7 +121,6 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::rt::time::sleep;
     use crate::service::{Service, ServiceFactory};
     use crate::util::lazy;
 
@@ -149,11 +129,7 @@ mod tests {
 
     #[crate::rt_test]
     async fn test_ka() {
-        let factory = KeepAlive::new(
-            Duration::from_millis(100),
-            LowResTime::with(Duration::from_millis(10)),
-            || TestErr,
-        );
+        let factory = KeepAlive::new(Millis(100), || TestErr);
         let _ = factory.clone();
 
         let service = factory.new_service(()).await.unwrap();
@@ -161,7 +137,7 @@ mod tests {
         assert_eq!(service.call(1usize).await, Ok(1usize));
         assert!(lazy(|cx| service.poll_ready(cx)).await.is_ready());
 
-        sleep(Duration::from_millis(500)).await;
+        sleep(Millis(500)).await;
         assert_eq!(
             lazy(|cx| service.poll_ready(cx)).await,
             Poll::Ready(Err(TestErr))
