@@ -1,13 +1,15 @@
-use std::{cell::RefCell, fmt, future::Future, pin::Pin, rc::Rc};
+use std::{
+    cell::RefCell, fmt, future::Future, marker::PhantomData, pin::Pin, rc::Rc, task,
+};
 
 use crate::http::Request;
 use crate::router::ResourceDef;
 use crate::service::boxed::{self, BoxServiceFactory};
-use crate::service::{apply, apply_fn_factory, pipeline_factory};
-use crate::service::{IntoServiceFactory, Service, ServiceFactory, Transform};
-use crate::util::{Either, Extensions, Ready};
+use crate::service::{map_config, pipeline_factory, PipelineFactory};
+use crate::service::{Identity, IntoServiceFactory, Service, ServiceFactory, Transform};
+use crate::util::{Extensions, Ready};
 
-use super::app_service::{AppEntry, AppFactory, AppRoutingFactory};
+use super::app_service::{AppFactory, AppService};
 use super::config::{AppConfig, ServiceConfig};
 use super::request::WebRequest;
 use super::resource::Resource;
@@ -24,11 +26,11 @@ type FnDataFactory =
 
 /// Application builder - structure that follows the builder pattern
 /// for building application instances.
-pub struct App<T, Err: ErrorRenderer = DefaultError> {
-    endpoint: T,
+pub struct App<M, F, Err: ErrorRenderer = DefaultError> {
+    middleware: M,
+    filter: PipelineFactory<F>,
     services: Vec<Box<dyn AppServiceFactory<Err>>>,
     default: Option<Rc<HttpNewService<Err>>>,
-    factory_ref: Rc<RefCell<Option<AppRoutingFactory<Err>>>>,
     data: Vec<Box<dyn DataFactory>>,
     data_factories: Vec<FnDataFactory>,
     external: Vec<ResourceDef>,
@@ -37,17 +39,16 @@ pub struct App<T, Err: ErrorRenderer = DefaultError> {
     case_insensitive: bool,
 }
 
-impl App<AppEntry<DefaultError>, DefaultError> {
+impl App<Identity, Filter<DefaultError>, DefaultError> {
     /// Create application builder. Application can be configured with a builder-like pattern.
     pub fn new() -> Self {
-        let fref = Rc::new(RefCell::new(None));
         App {
-            endpoint: AppEntry::new(fref.clone()),
+            middleware: Identity,
+            filter: pipeline_factory(Filter::new()),
             data: Vec::new(),
             data_factories: Vec::new(),
             services: Vec::new(),
             default: None,
-            factory_ref: fref,
             external: Vec::new(),
             extensions: Extensions::new(),
             error_renderer: DefaultError,
@@ -56,17 +57,16 @@ impl App<AppEntry<DefaultError>, DefaultError> {
     }
 }
 
-impl<Err: ErrorRenderer> App<AppEntry<Err>, Err> {
+impl<Err: ErrorRenderer> App<Identity, Filter<Err>, Err> {
     /// Create application builder with custom error renderer.
     pub fn with(err: Err) -> Self {
-        let fref = Rc::new(RefCell::new(None));
         App {
-            endpoint: AppEntry::new(fref.clone()),
+            middleware: Identity,
+            filter: pipeline_factory(Filter::new()),
             data: Vec::new(),
             data_factories: Vec::new(),
             services: Vec::new(),
             default: None,
-            factory_ref: fref,
             external: Vec::new(),
             extensions: Extensions::new(),
             error_renderer: err,
@@ -75,12 +75,12 @@ impl<Err: ErrorRenderer> App<AppEntry<Err>, Err> {
     }
 }
 
-impl<T, Err> App<T, Err>
+impl<M, T, Err> App<M, T, Err>
 where
     T: ServiceFactory<
         Config = (),
         Request = WebRequest<Err>,
-        Response = WebResponse,
+        Response = WebRequest<Err>,
         Error = Err::Container,
         InitError = (),
     >,
@@ -355,44 +355,37 @@ where
     ///         .route("/index.html", web::get().to(index));
     /// }
     /// ```
-    pub fn filter<F>(
+    pub fn filter<S, U>(
         self,
-        filter: F,
+        filter: U,
     ) -> App<
+        M,
         impl ServiceFactory<
             Config = (),
             Request = WebRequest<Err>,
-            Response = WebResponse,
+            Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         >,
         Err,
     >
     where
-        F: ServiceFactory<
+        S: ServiceFactory<
             Config = (),
             Request = WebRequest<Err>,
-            Response = Either<WebRequest<Err>, WebResponse>,
+            Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         >,
+        U: IntoServiceFactory<S>,
     {
-        let ep = self.endpoint;
-        let endpoint =
-            pipeline_factory(filter).and_then_apply_fn(ep, move |result, srv| {
-                match result {
-                    Either::Left(req) => Either::Left(srv.call(req)),
-                    Either::Right(res) => Either::Right(Ready::Ok(res)),
-                }
-            });
-
         App {
-            endpoint,
+            filter: self.filter.and_then(filter.into_factory()),
+            middleware: self.middleware,
             data: self.data,
             data_factories: self.data_factories,
             services: self.services,
             default: self.default,
-            factory_ref: self.factory_ref,
             external: self.external,
             extensions: self.extensions,
             error_renderer: self.error_renderer,
@@ -410,10 +403,7 @@ where
     ///
     /// Notice that the keyword for registering middleware is `wrap`. As you
     /// register middleware using `wrap` in the App builder,  imagine wrapping
-    /// layers around an inner App.  The first middleware layer exposed to a
-    /// Request is the outermost layer-- the *last* registered in
-    /// the builder chain.  Consequently, the *first* middleware registered
-    /// in the builder chain is the *last* to execute during request processing.
+    /// layers around an inner App.
     ///
     /// ```rust
     /// use ntex::http::header::{CONTENT_TYPE, HeaderValue};
@@ -429,96 +419,14 @@ where
     ///         .route("/index.html", web::get().to(index));
     /// }
     /// ```
-    pub fn wrap<M>(
-        self,
-        mw: M,
-    ) -> App<
-        impl ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-        Err,
-    >
-    where
-        M: Transform<
-            T::Service,
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-    {
+    pub fn wrap<U>(self, mw: U) -> App<Stack<M, U>, T, Err> {
         App {
-            endpoint: apply(mw, self.endpoint),
+            middleware: Stack::new(self.middleware, mw),
+            filter: self.filter,
             data: self.data,
             data_factories: self.data_factories,
             services: self.services,
             default: self.default,
-            factory_ref: self.factory_ref,
-            external: self.external,
-            extensions: self.extensions,
-            error_renderer: self.error_renderer,
-            case_insensitive: self.case_insensitive,
-        }
-    }
-
-    /// Registers middleware, in the form of a closure, that runs during inbound
-    /// and/or outbound processing in the request lifecycle (request -> response),
-    /// modifying request/response as necessary, across all requests managed by
-    /// the *Application*.
-    ///
-    /// Use middleware when you need to read or modify *every* request or response in some way.
-    ///
-    /// ```rust
-    /// use ntex::{web, Service};
-    /// use ntex::http::header::{CONTENT_TYPE, HeaderValue};
-    ///
-    /// async fn index() -> &'static str {
-    ///     "Welcome!"
-    /// }
-    ///
-    /// fn main() {
-    ///     let app = web::App::new()
-    ///         .wrap_fn(|req, srv| {
-    ///             let fut = srv.call(req);
-    ///             async {
-    ///                 let mut res = fut.await?;
-    ///                 res.headers_mut().insert(
-    ///                    CONTENT_TYPE, HeaderValue::from_static("text/plain"),
-    ///                 );
-    ///                 Ok(res)
-    ///             }
-    ///         })
-    ///         .route("/index.html", web::get().to(index));
-    /// }
-    /// ```
-    pub fn wrap_fn<F, R>(
-        self,
-        mw: F,
-    ) -> App<
-        impl ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-        Err,
-    >
-    where
-        F: Fn(WebRequest<Err>, &T::Service) -> R + Clone,
-        R: Future<Output = Result<WebResponse, Err::Container>>,
-    {
-        App {
-            endpoint: apply_fn_factory(self.endpoint, mw),
-            data: self.data,
-            data_factories: self.data_factories,
-            services: self.services,
-            default: self.default,
-            factory_ref: self.factory_ref,
             external: self.external,
             extensions: self.extensions,
             error_renderer: self.error_renderer,
@@ -533,7 +441,26 @@ where
         self.case_insensitive = true;
         self
     }
+}
 
+impl<M, F, Err> App<M, F, Err>
+where
+    M: Transform<AppService<F::Service, Err>> + 'static,
+    M::Service: Service<
+        Request = WebRequest<Err>,
+        Response = WebResponse,
+        Error = Err::Container,
+    >,
+    F: ServiceFactory<
+        Config = (),
+        Request = WebRequest<Err>,
+        Response = WebRequest<Err>,
+        Error = Err::Container,
+        InitError = (),
+    >,
+    F::Future: 'static,
+    Err: ErrorRenderer,
+{
     /// Construct service factory with default `AppConfig`, suitable for `http::HttpService`.
     ///
     /// ```rust,no_run
@@ -558,10 +485,10 @@ where
         Config = (),
         Request = Request,
         Response = WebResponse,
-        Error = T::Error,
-        InitError = T::InitError,
+        Error = Err::Container,
+        InitError = (),
     > {
-        crate::map_config(self.into_factory(), move |_| Default::default())
+        map_config(self.into_factory(), move |_| Default::default())
     }
 
     /// Construct service factory suitable for `http::HttpService`.
@@ -589,52 +516,124 @@ where
         Config = (),
         Request = Request,
         Response = WebResponse,
-        Error = T::Error,
-        InitError = T::InitError,
+        Error = Err::Container,
+        InitError = (),
     > {
-        crate::map_config(self.into_factory(), move |_| cfg.clone())
+        map_config(self.into_factory(), move |_| cfg.clone())
     }
 }
 
-impl<T, Err> IntoServiceFactory<AppFactory<T, Err>> for App<T, Err>
+impl<M, F, Err> IntoServiceFactory<AppFactory<M, F, Err>> for App<M, F, Err>
 where
-    T: ServiceFactory<
-        Config = (),
+    M: Transform<AppService<F::Service, Err>> + 'static,
+    M::Service: Service<
         Request = WebRequest<Err>,
         Response = WebResponse,
         Error = Err::Container,
+    >,
+    F: ServiceFactory<
+        Config = (),
+        Request = WebRequest<Err>,
+        Response = WebRequest<Err>,
+        Error = Err::Container,
         InitError = (),
     >,
-    T::Future: 'static,
+    F::Future: 'static,
     Err: ErrorRenderer,
 {
-    fn into_factory(self) -> AppFactory<T, Err> {
+    fn into_factory(self) -> AppFactory<M, F, Err> {
         AppFactory {
+            filter: self.filter,
+            middleware: Rc::new(self.middleware),
             data: Rc::new(self.data),
             data_factories: Rc::new(self.data_factories),
-            endpoint: self.endpoint,
             services: Rc::new(RefCell::new(self.services)),
             external: RefCell::new(self.external),
             default: self.default,
-            factory_ref: self.factory_ref,
             extensions: RefCell::new(Some(self.extensions)),
             case_insensitive: self.case_insensitive,
         }
     }
 }
 
+pub struct Stack<Inner, Outer> {
+    inner: Inner,
+    outer: Outer,
+}
+
+impl<Inner, Outer> Stack<Inner, Outer> {
+    pub(super) fn new(inner: Inner, outer: Outer) -> Self {
+        Stack { inner, outer }
+    }
+}
+
+impl<S, Inner, Outer> Transform<S> for Stack<Inner, Outer>
+where
+    Inner: Transform<S>,
+    Outer: Transform<Inner::Service>,
+{
+    type Service = Outer::Service;
+
+    fn new_transform(&self, service: S) -> Self::Service {
+        self.outer.new_transform(self.inner.new_transform(service))
+    }
+}
+
+pub struct Filter<Err>(PhantomData<Err>);
+
+impl<Err: ErrorRenderer> Filter<Err> {
+    pub(super) fn new() -> Self {
+        Filter(PhantomData)
+    }
+}
+
+impl<Err: ErrorRenderer> ServiceFactory for Filter<Err> {
+    type Config = ();
+    type Request = WebRequest<Err>;
+    type Response = WebRequest<Err>;
+    type Error = Err::Container;
+    type InitError = ();
+    type Service = Filter<Err>;
+    type Future = Ready<Filter<Err>, ()>;
+
+    #[inline]
+    fn new_service(&self, _: ()) -> Self::Future {
+        Ready::Ok(Filter(PhantomData))
+    }
+}
+
+impl<Err: ErrorRenderer> Service for Filter<Err> {
+    type Request = WebRequest<Err>;
+    type Response = WebRequest<Err>;
+    type Error = Err::Container;
+    type Future = Ready<WebRequest<Err>, Err::Container>;
+
+    #[inline]
+    fn poll_ready(
+        &self,
+        _: &mut task::Context<'_>,
+    ) -> task::Poll<Result<(), Self::Error>> {
+        task::Poll::Ready(Ok(()))
+    }
+
+    #[inline]
+    fn call(&self, req: Self::Request) -> Self::Future {
+        Ready::Ok(req)
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use bytes::Bytes;
-
     use super::*;
     use crate::http::header::{self, HeaderValue};
     use crate::http::{Method, StatusCode};
-    use crate::web::middleware::DefaultHeaders;
-    use crate::web::request::WebRequest;
+    use crate::service::{fn_service, Service};
+    use crate::util::{Bytes, Ready};
     use crate::web::test::{call_service, init_service, read_body, TestRequest};
-    use crate::web::{self, DefaultError, HttpRequest, HttpResponse};
-    use crate::{fn_service, Service};
+    use crate::web::{
+        self, middleware::DefaultHeaders, request::WebRequest, DefaultError,
+        HttpRequest, HttpResponse,
+    };
 
     #[crate::rt_test]
     async fn test_default_resource() {
@@ -727,17 +726,21 @@ mod tests {
 
     #[crate::rt_test]
     async fn test_filter() {
+        let filter = Rc::new(std::cell::Cell::new(false));
+        let filter2 = filter.clone();
         let srv = init_service(
             App::new()
-                .filter(fn_service(|req: WebRequest<_>| async move {
-                    Ok(Either::Right(req.into_response(HttpResponse::NotFound())))
+                .filter(fn_service(move |req: WebRequest<_>| {
+                    filter2.set(true);
+                    Ready::Ok(req)
                 }))
                 .route("/test", web::get().to(|| async { HttpResponse::Ok() })),
         )
         .await;
         let req = TestRequest::with_uri("/test").to_request();
         let resp = call_service(&srv, req).await;
-        assert_eq!(resp.status(), StatusCode::NOT_FOUND);
+        assert_eq!(resp.status(), StatusCode::OK);
+        assert!(filter.get());
     }
 
     #[crate::rt_test]
@@ -780,60 +783,6 @@ mod tests {
         );
     }
 
-    #[crate::rt_test]
-    async fn test_wrap_fn() {
-        let srv = init_service(
-            App::new()
-                .wrap_fn(|req, srv| {
-                    let fut = srv.call(req);
-                    async move {
-                        let mut res = fut.await?;
-                        res.headers_mut().insert(
-                            header::CONTENT_TYPE,
-                            HeaderValue::from_static("0001"),
-                        );
-                        Ok(res)
-                    }
-                })
-                .service(web::resource("/test").to(|| async { HttpResponse::Ok() })),
-        )
-        .await;
-        let req = TestRequest::with_uri("/test").to_request();
-        let resp = call_service(&srv, req).await;
-        assert_eq!(resp.status(), StatusCode::OK);
-        assert_eq!(
-            resp.headers().get(header::CONTENT_TYPE).unwrap(),
-            HeaderValue::from_static("0001")
-        );
-    }
-
-    #[crate::rt_test]
-    async fn test_router_wrap_fn() {
-        let srv = init_service(
-            App::new()
-                .route("/test", web::get().to(|| async { HttpResponse::Ok() }))
-                .wrap_fn(|req, srv| {
-                    let fut = srv.call(req);
-                    async {
-                        let mut res = fut.await?;
-                        res.headers_mut().insert(
-                            header::CONTENT_TYPE,
-                            HeaderValue::from_static("0001"),
-                        );
-                        Ok(res)
-                    }
-                }),
-        )
-        .await;
-        let req = TestRequest::with_uri("/test").to_request();
-        let resp = call_service(&srv, req).await;
-        assert_eq!(resp.status(), StatusCode::OK);
-        assert_eq!(
-            resp.headers().get(header::CONTENT_TYPE).unwrap(),
-            HeaderValue::from_static("0001")
-        );
-    }
-
     #[crate::rt_test]
     async fn test_case_insensitive_router() {
         let srv = init_service(
@@ -851,6 +800,7 @@ mod tests {
         assert_eq!(resp.status(), StatusCode::OK);
     }
 
+    #[cfg(feature = "url")]
     #[crate::rt_test]
     async fn test_external_resource() {
         let srv = init_service(
