@@ -1,16 +1,13 @@
 //! Various helpers for ntex applications to use during testing.
 use std::{
     convert::TryFrom, error::Error, fmt, net, net::SocketAddr, rc::Rc, sync::mpsc,
-    thread, time,
+    thread,
 };
 
-use bytes::{Bytes, BytesMut};
-use futures_core::Stream;
-use serde::de::DeserializeOwned;
-use serde::Serialize;
-
 #[cfg(feature = "cookie")]
 use coo_kie::Cookie;
+use serde::de::DeserializeOwned;
+use serde::Serialize;
 
 use crate::codec::{AsyncRead, AsyncWrite};
 use crate::http::body::MessageBody;
@@ -21,17 +18,18 @@ use crate::http::header::{HeaderName, HeaderValue, CONTENT_TYPE};
 use crate::http::test::TestRequest as HttpTestRequest;
 use crate::http::{HttpService, Method, Payload, Request, StatusCode, Uri, Version};
 use crate::router::{Path, ResourceDef};
-use crate::rt::{time::sleep, System};
-use crate::server::Server;
-use crate::util::{next, Extensions, Ready};
-use crate::{map_config, IntoService, IntoServiceFactory, Service, ServiceFactory};
+use crate::service::{
+    map_config, IntoService, IntoServiceFactory, Service, ServiceFactory,
+};
+use crate::time::{sleep, Millis, Seconds};
+use crate::util::{next, Bytes, BytesMut, Extensions, Ready};
+use crate::{rt::System, server::Server, Stream};
 
 use crate::web::config::AppConfig;
-use crate::web::dev::{WebRequest, WebResponse};
 use crate::web::error::{DefaultError, ErrorRenderer};
 use crate::web::httprequest::{HttpRequest, HttpRequestPool};
 use crate::web::rmap::ResourceMap;
-use crate::web::{FromRequest, HttpResponse, Responder};
+use crate::web::{FromRequest, HttpResponse, Responder, WebRequest, WebResponse};
 
 /// Create service that always responds with `HttpResponse::Ok()`
 pub fn ok_service<Err: ErrorRenderer>() -> impl Service<
@@ -130,8 +128,7 @@ where
 /// Helper function that returns a response body of a TestRequest
 ///
 /// ```rust
-/// use bytes::Bytes;
-/// use ntex::http::header;
+/// use ntex::{http::header, util::Bytes};
 /// use ntex::web::{self, test, App, HttpResponse};
 ///
 /// #[ntex::test]
@@ -173,8 +170,7 @@ where
 /// Helper function that returns a response body of a WebResponse.
 ///
 /// ```rust
-/// use bytes::Bytes;
-/// use ntex::http::header;
+/// use ntex::{util::Bytes, http::header};
 /// use ntex::web::{self, test, App, HttpResponse};
 ///
 /// #[ntex::test]
@@ -736,25 +732,25 @@ where
                     .set_alpn_protos(b"\x02h2\x08http/1.1")
                     .map_err(|e| log::error!("Cannot set alpn protocol: {:?}", e));
                 Connector::default()
-                    .lifetime(time::Duration::from_secs(0))
-                    .keep_alive(time::Duration::from_millis(30000))
-                    .timeout(time::Duration::from_millis(30000))
-                    .disconnect_timeout(time::Duration::from_millis(3000))
+                    .lifetime(Seconds::ZERO)
+                    .keep_alive(Seconds(30))
+                    .timeout(Millis(30_000))
+                    .disconnect_timeout(Millis(3_000))
                     .openssl(builder.build())
                     .finish()
             }
             #[cfg(not(feature = "openssl"))]
             {
                 Connector::default()
-                    .lifetime(time::Duration::from_secs(0))
-                    .timeout(time::Duration::from_millis(30000))
+                    .lifetime(Seconds::ZERO)
+                    .timeout(Millis(30_000))
                     .finish()
             }
         };
 
         Client::build()
             .connector(connector)
-            .timeout(time::Duration::from_millis(30000))
+            .timeout(Seconds(30))
             .finish()
     };
 
@@ -772,7 +768,7 @@ where
 pub struct TestServerConfig {
     tp: HttpVer,
     stream: StreamType,
-    client_timeout: u16,
+    client_timeout: Seconds,
 }
 
 #[derive(Clone, Debug)]
@@ -820,7 +816,7 @@ impl TestServerConfig {
         TestServerConfig {
             tp: HttpVer::Both,
             stream: StreamType::Tcp,
-            client_timeout: 5000,
+            client_timeout: Seconds(5),
         }
     }
 
@@ -851,7 +847,7 @@ impl TestServerConfig {
     }
 
     /// Set server client timeout in seconds for first request.
-    pub fn client_timeout(mut self, val: u16) -> Self {
+    pub fn client_timeout(mut self, val: Seconds) -> Self {
         self.client_timeout = val;
         self
     }
@@ -952,7 +948,7 @@ impl TestServer {
     pub async fn stop(self) {
         self.server.stop(true).await;
         self.system.stop();
-        sleep(time::Duration::from_millis(100)).await;
+        sleep(Millis(100)).await;
     }
 }
 
@@ -1226,7 +1222,7 @@ mod tests {
                     ))
                     .finish(),
             )
-            .timeout(time::Duration::from_millis(30000))
+            .timeout(Seconds(30))
             .finish();
 
         let url = format!("https://localhost:{}/", srv.addr.port());
